@@ -1,21 +1,29 @@
-#' Visually comparing shrinkage and LS estimates
+#'Visually comparing shrinkage and LS estimates
 #'
-#' This function creates a plot (using \code{qplot()}) where the shrinkage estimate appears on the
-#' horizontal axis and the LS estimate appears on the vertical axis.
+#'This function creates a plot (using \code{qplot()}) where the shrinkage
+#'estimate appears on the horizontal axis and the LS estimate appears on the
+#'vertical axis.
+#'
+#'
+#'@param eb a matrix of random effects
+#'@param ols a matrix of the OLS estimates found using \code{random_ls_coef}
+#'@param identify the percentage of points to identify as unusual,
+#'\code{FALSE} if you do not want the points identified.
+#'@param silent logical: should the list of data frames used to make the plots
+#' be supressed.
+#'@param ... other arguments to be passed to \code{qplot()}
+#'@author Adam Loy \email{aloy@@istate.edu}
+#'@export
+#'@examples
+#'
+#'wages.fm1 <- lmer(lnw ~ exper + (exper | id), data = wages)
+#'wages.sepLM <- adjust_lmList(lnw ~ exper | id, data = wages)
+#'rancoef.eb <- coef(wages.fm1)$id
+#'rancoef.ols <- coef(wages.sepLM)
+#'compare_eb_ls(eb = rancoef.eb, ols = rancoef.ols, identify = 0.01)
 #'
-#' @param eb a matrix of random effects
-#' @param ols a matrix of the LS estimates found using \code{random_ls_coef}
-#' @param identify  identify the percentage of points to identify as unusual, \code{FALSE} if you do not want the points identified.
-#' @param ... other arguments to be passed to \code{qplot()}
-#' @author Adam Loy \email{aloy@@istate.edu}
-#' @examples
-#' wages.fm1 <- lmer(lnw ~ exper + (exper | id), data = wages)
-#' wages.sepLM <- adjust_lmList(lnw ~ exper | id, data = wages)
-#' rancoef.eb <- coef(wages.fm1)$id
-#' rancoef.ols <- coef(wages.sepLM)
-#' compare_eb_ls(eb = rancoef.eb, ols = rancoef.ols, identify = 0.01)
-#' @export
 compare_eb_ls <- function(eb, ols, identify = FALSE, silent = TRUE, ...){
+  unusual <- ids <- NULL # Make codetools happy
 	ret <- NULL
 	for(i in 1:dim(ols)[2]){
 	p <- qplot(x = eb[,i], y = ols[,i], geom = "point", main = colnames(eb)[i], 
