@@ -18,7 +18,7 @@ covratio <- function(object, ...){
 }
 
 #' @export
-#' @rdname covratio.mer
+#' @rdname covratio
 #' @method covratio default
 #' @S3method covratio default
 covratio.default <- function(object, ...){
@@ -32,7 +32,7 @@ covtrace <- function(object, ...){
 }
 
 #' @export
-#' @rdname covratio.mer
+#' @rdname covratio
 #' @method covtrace default
 #' @S3method covtrace default
 covtrace.default <- function(object, ...){
@@ -46,7 +46,7 @@ mdffits <- function(object, ...){
 }
 
 #' @export
-#' @rdname cooks.distance.mer
+#' @rdname cooks.distance
 #' @method mdffits default
 #' @S3method mdffits default
 mdffits.default <- function(object, ...){
@@ -78,7 +78,7 @@ rvc.default <- function(object, ...){
 #' @method leverage mer
 #' @S3method leverage mer
 #' @aliases leverage
-#' @param object fitted object of class \code{mer}
+#' @param object fitted object of class \code{mer} of \code{lmerMod}
 #' @param level the level at which the leverage should be calculated: either
 #'   1 for observation level leverage or the name of the grouping factor 
 #'   (as defined in \code{flist} of the \code{mer} object) for group level
@@ -195,6 +195,69 @@ leverage.mer <- function(object, level, ...) {
   if(level != 1) return(grp.lev)
 }
 
+#' @export
+#' @rdname leverage.mer
+#' @method leverage lmerMod
+#' @S3method leverage lmerMod
+leverage.lmerMod <- function(object, level, ...) {
+  if(!isNestedModel(object)) {
+    stop("leverage.mer has not yet been implemented for models with 
+         crossed random effects")
+  }
+  if(!level %in% c( 1, names(getME(object, "flist")))) {
+    stop("level can only be 1 or a grouping factor from the fitted model.")
+  }
+  
+  if(!isLMM(object)){
+    stop("leverage is currently not implemented for GLMMs or NLMMs.")
+  }
+  
+  X <- getME(object, "X")
+  # Z <- BlockZ(object)
+  
+  n     <- nrow(X)
+#  nt    <- object@dims[["nt"]]  # number of random-effects terms in the model
+  ngrps <- unname( summary(object)$ngrps )
+  
+  vc   <- VarCorr(object)
+  # D  <- kronecker( Diagonal(ngrps), bdiag(vc) )
+  ZDZt <- attr(vc, "sc")^2 * crossprod( getME(object, "A") )
+  R    <- Diagonal( n = n, x = attr(vc, "sc")^2 )
+  
+  V      <- ZDZt + R
+  V.chol <- chol( V )
+  Vinv   <- chol2inv( V.chol )
+  
+  xvix.inv <- attr(vc, "sc")^2 * chol2inv(getME(object, "RX"))
+  
+  H1 <- X %*% xvix.inv %*% t(X) %*% Vinv
+  H2 <- ZDZt %*% Vinv %*% (Diagonal( n = n ) - H1)
+  
+  diag.H1 <- diag(H1)
+  diag.H2 <- diag(H2)
+  diag.H2.uc <- diag(ZDZt) / attr(vc, "sc")^2
+  
+  if(level == 1) {
+    lev1 <- data.frame(overall = diag.H1 + diag.H2, fixef = diag.H1, 
+                       ranef =  diag.H2, ranef.uc = diag.H2.uc)
+    #     class(lev1) <- "leverage"
+  } else {
+    flist   <- data.frame( getME(object, "flist")[[level]] )
+    
+    grp.lev.fixef <- aggregate(diag.H1, flist, mean)[,2]
+    grp.lev.ranef <- aggregate(diag.H2, flist, mean)[,2]
+    grp.lev.ranef.uc <- aggregate(diag.H2.uc, flist, mean)[,2]
+    
+    grp.lev <- data.frame( overall = grp.lev.fixef + grp.lev.ranef,
+                           fixef = grp.lev.fixef, 
+                           ranef = grp.lev.ranef,
+                           ranef.uc = grp.lev.ranef.uc)
+    #     class(grp.lev) <- "leverage"
+  }
+  
+  if(level == 1) return(lev1)
+  if(level != 1) return(grp.lev)
+  }
 
 #' Influence on fixed effects of HLMs
 #'
@@ -219,10 +282,11 @@ leverage.mer <- function(object, level, ...) {
 #' the difference between the full and deleted parameter estimates.
 #'
 #'@export
+#'@rdname cooks.distance
 #'@method cooks.distance mer
 #'@S3method cooks.distance mer
 #'@aliases cooks.distance
-#'@param model fitted model of class \code{mer}
+#'@param model fitted model of class \code{mer} or \code{lmerMod}
 #'@param group variable used to define the group for which cases will be
 #'deleted.  If \code{group = NULL}, then individual cases will be deleted.
 #'@param delete index of individual cases to be deleted. To delete specific 
@@ -329,6 +393,71 @@ cooks.distance.mer <- function(model, group = NULL, delete = NULL, ...) {
   return(res)
 }
 
+#' @export
+#' @rdname cooks.distance
+#' @method cooks.distance lmerMod
+#' @S3method cooks.distance lmerMod
+cooks.distance.lmerMod <- function(model, group = NULL, delete = NULL, ...) {
+  if(!is.null(group)) {
+    if(!group %in% names(getME(model, "flist"))) {
+      stop(paste(group, "is not a valid grouping factor for this model."))
+    }
+  }
+  if(!isLMM(model)){
+    stop("cooks.distance is currently not implemented for GLMMs or NLMMs.")
+  }
+  
+  XVXinv <- X <- Vinv <- Y <- NULL # Make codetools happy
+  
+  # Extract key pieces of the model
+  mats <- .lmerMod_matrices(model)
+  
+  betaHat <- with(mats, XVXinv %*% t(X) %*% Vinv %*% Y)
+  
+  # Obtaining the building blocks
+  if(is.null(group) & is.null(delete)) {
+    calc.cooksd <- .Call("cooksdObs", y_ = mats$Y, X_ = as.matrix(mats$X), 
+                         Vinv_ = as.matrix(mats$Vinv), 
+                         XVXinv_ = as.matrix(mats$XVXinv), 
+                         beta_ = as.matrix(betaHat), PACKAGE = "HLMdiag")
+    res <- calc.cooksd[[1]]
+    attr(res, "beta_cdd") <- calc.cooksd[[2]]
+  }
+  
+  else{
+    e <- with(mats, Y - X %*% betaHat)
+    
+    if( !is.null(group) ){
+      grp.names <- unique( mats$flist[[group]] )
+      
+      if( is.null(delete) ){
+        del.index <- lapply(1:mats$ngrps[group], 
+                            function(x) {
+                              ind <- which(mats$flist[[group]] == grp.names[x]) - 1
+                            })
+      } else{
+        del.index <- list( which(mats$flist[[group]] %in% delete) - 1 )
+      }
+    } else{
+      del.index <- list( delete - 1 )
+    }
+    
+    calc.cooksd <- .Call("cooksdSubset", index = del.index, 
+                         X_ = as.matrix(mats$X), 
+                         P_ = as.matrix(mats$P), 
+                         Vinv_ = as.matrix(mats$Vinv), 
+                         XVXinv_ = as.matrix(mats$XVXinv), 
+                         e_ = as.numeric(e), PACKAGE = "HLMdiag")
+    
+    res <- calc.cooksd[[1]]
+    attr(res, "beta_cdd") <- calc.cooksd[[2]] 
+  }
+  
+  class(res) <- "fixef.dd"
+  return(res)
+}
+
+
 print.fixef.dd <- function(x, ...) {
   attributes(x) <- NULL
   print(x, ...)
@@ -338,11 +467,11 @@ print.vcov.dd <- function(x, ...) { print(unclass(x), ...) }
 
 
 #' @export
-#' @rdname cooks.distance.mer
+#' @rdname cooks.distance
 #' @method mdffits mer
 #' @S3method mdffits mer
 #' @aliases mdffits
-#' @param object fitted object of class \code{mer}
+#' @param object fitted object of class \code{mer} or \code{lmerMod}
 #' @examples
 #' 
 #' # MDFFITS  for individual observations
@@ -407,6 +536,56 @@ mdffits.mer <- function(object, group = NULL, delete = NULL, ...) {
 }
 
 
+#' @export
+#' @rdname cooks.distance
+#' @method mdffits lmerMod
+#' @S3method mdffits lmerMod
+mdffits.lmerMod <- function(object, group = NULL, delete = NULL, ...) {
+  if(!is.null(group)) {
+    if(!group %in% names(getME(object, "flist"))) {
+      stop(paste(group, "is not a valid grouping factor for this model."))
+    }
+  }
+  if(!isLMM(object)){
+    stop("mdffits is currently not implemented for GLMMs or NLMMs.")
+  }
+  
+  XVXinv <- X <- Vinv <- Y <- NULL # Make codetools happy
+  
+  # Extract key pieces of the model
+  mats <- .lmerMod_matrices(object)
+  
+  betaHat <- with(mats, XVXinv %*% t(X) %*% Vinv %*% Y)
+  e <- with(mats, Y - X %*% betaHat)
+  
+  if( !is.null(group) ){
+    grp.names <- unique( mats$flist[[group]] )
+    
+    if( is.null(delete) ){
+      del.index <- lapply(1:mats$ngrps[group], 
+                          function(x) {
+                            ind <- which(mats$flist[[group]] == grp.names[x]) - 1
+                          })
+    } else{
+      del.index <- list( which(mats$flist[[group]] %in% delete) - 1 )
+    }
+  } else{
+    if( is.null(delete) ){
+      del.index <- split(0:(mats$n-1), 0:(mats$n-1))
+    } else { del.index <- list( delete - 1 ) }
+  }
+  
+  calc.mdffits <- .Call("mdffitsSubset", index = del.index, X_ = mats$X, 
+                        P_ = mats$P, Vinv_ = as.matrix(mats$Vinv), 
+                        XVXinv_ = as.matrix(mats$XVXinv), 
+                        e_ = as.numeric(e), PACKAGE = "HLMdiag")
+  res <- calc.mdffits[[1]]
+  attr(res, "beta_cdd") <- calc.mdffits[[2]] 
+  
+  class(res) <- "fixef.dd"
+  return(res)
+}
+
 #' Influence on precision of fixed effects in HLMs
 #'
 #' These functions calculate measures of the change in the covariance
@@ -423,10 +602,11 @@ mdffits.mer <- function(object, group = NULL, delete = NULL, ...) {
 #'  compares the trace of the ratio. 
 #'  
 #'@export
+#' @rdname covratio
 #'@method covratio mer
 #'@S3method covratio mer
 #'@aliases covratio
-#'@param object fitted object of class \code{mer}
+#'@param object fitted object of class \code{mer} or \code{lmerMod}
 #'@param group variable used to define the group for which cases will be
 #'deleted.  If \code{group = NULL}, then individual cases will be deleted.
 #'@param delete index of individual cases to be deleted. To delete specific 
@@ -515,9 +695,50 @@ covratio.mer <- function(object, group = NULL, delete = NULL, ...) {
 }
 
 
+#'@export
+#'@rdname covratio
+#'@method covratio lmerMod
+#'@S3method covratio lmerMod
+covratio.lmerMod <- function(object, group = NULL, delete = NULL, ...) {
+  if(!is.null(group)) {
+    if(!group %in% names(getME(object, "flist"))) {
+      stop(paste(group, "is not a valid grouping factor for this model."))
+    }
+  }
+  if(!isLMM(object)){
+    stop("covratio is currently not implemented for GLMMs or NLMMs.")
+  }
+  
+  # Extract key pieces of the model
+  mats <- .lmerMod_matrices(object)
+  
+  if( !is.null(group) ){
+    grp.names <- unique( mats$flist[[group]] )
+    
+    if( is.null(delete) ){
+      del.index <- lapply(1:mats$ngrps[group], 
+                          function(x) {
+                            ind <- which(mats$flist[[group]] == grp.names[x]) - 1
+                          })
+    } else{
+      del.index <- list( which(mats$flist[[group]] %in% delete) - 1 )
+    }
+  } else{
+    if( is.null(delete) ){
+      del.index <- split(0:(mats$n-1), 0:(mats$n-1))
+    } else { del.index <- list( delete - 1 ) }
+  }
+  
+  res <- .Call("covratioCalc", index = del.index, X_ = mats$X, P_ = mats$P, 
+               Vinv_ = as.matrix(mats$Vinv), XVXinv_ = as.matrix(mats$XVXinv), 
+               PACKAGE = "HLMdiag")
+  
+  class(res) <- "vcov.dd"
+  return(res)
+}
 
 #'@export
-#'@rdname covratio.mer
+#'@rdname covratio
 #'@method covtrace mer
 #'@S3method covtrace mer
 #'@aliases covtrace
@@ -575,6 +796,48 @@ covtrace.mer <- function(object, group = NULL, delete = NULL, ...) {
   return(res)
 }
 
+
+#'@export
+#'@rdname covratio
+#'@method covtrace lmerMod
+#'@S3method covtrace lmerMod
+covtrace.lmerMod <- function(object, group = NULL, delete = NULL, ...) {
+  if(!is.null(group)) {
+    if(!group %in% names(getME(object, "flist"))) {
+      stop(paste(group, "is not a valid grouping factor for this model."))
+    }
+  }
+  if(!isLMM(object)){
+    stop("covtrace is currently not implemented for GLMMs or NLMMs.")
+  }
+  
+  # Extract key pieces of the model
+  mats <- .lmerMod_matrices(object)
+  
+  if( !is.null(group) ){
+    grp.names <- unique( mats$flist[[group]] )
+    
+    if( is.null(delete) ){
+      del.index <- lapply(1:mats$ngrps[group], 
+                          function(x) {
+                            ind <- which(mats$flist[[group]] == grp.names[x]) - 1
+                          })
+    } else{
+      del.index <- list( which(mats$flist[[group]] %in% delete) - 1 )
+    }
+  } else{
+    if( is.null(delete) ){
+      del.index <- split(0:(mats$n-1), 0:(mats$n-1))
+    } else { del.index <- list( delete - 1 ) }
+  }
+  
+  res <- .Call("covtraceCalc", index = del.index, X_ = mats$X, P_ = mats$P, 
+               Vinv_ = as.matrix(mats$Vinv), XVXinv_ = as.matrix(mats$XVXinv), 
+               PACKAGE = "HLMdiag")
+  
+  return(res)
+}
+
 #' Relative variance change for HLMs
 #' 
 #' This function calculates the relative variance change (RVC) of
@@ -584,7 +847,7 @@ covtrace.mer <- function(object, group = NULL, delete = NULL, ...) {
 #' @method rvc mer
 #' @S3method rvc mer
 #' @aliases rvc
-#'@param object fitted object of class \code{mer}
+#'@param object fitted object of class \code{mer} or \code{lmerMod}
 #'@param group variable used to define the group for which cases will be
 #'deleted.  If \code{group = NULL}, then individual cases will be deleted.
 #'@param delete index of individual cases to be deleted. To delete specific 
@@ -615,3 +878,13 @@ rvc.mer <- function(object, group = NULL, delete = NULL, ...) {
     delete <- case_delete(object, group = group, type = "varcomp", delete = delete)
     return( rvc(delete) )
 }
+
+
+#' @export
+#' @rdname rvc.mer
+#' @method rvc lmerMod
+#' @S3method rvc lmerMod
+rvc.lmerMod <- function(object, group = NULL, delete = NULL, ...) {
+  delete <- case_delete(object, group = group, type = "varcomp", delete = delete)
+  return( rvc(delete) )
+}
\ No newline at end of file
